package edu.harvard.mgh.purcell.gplink.base_form;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import java.io.File;

import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.event.DocumentListener;
import javax.swing.filechooser.FileFilter;

import edu.harvard.mgh.purcell.gplink.mainGUI.*;
/**
 * General useful functions and classes that are indepedent of our
 * data structure.
 * @author Kathe Todd-Brown
 *
 */
public class GeneralUtil {

	/**
	 * UpdatingTextField is a wrapper for a text field with 
	 * a listener.
	 * @author Kathe Todd-Brown
	 *
	 */
	@SuppressWarnings("serial")
	public static class UpdatingTextField extends JTextField{
		public UpdatingTextField(String s, int column,
				DocumentListener updateOK_DL){
			super(s, column);
			this.setEditable(false);
			this.getDocument().addDocumentListener(updateOK_DL);
		}
		
		public UpdatingTextField(int column,
				DocumentListener updateOK_DL){
			super(column);
			this.setEditable(false);
			this.getDocument().addDocumentListener(updateOK_DL);
		}
		
		public UpdatingTextField(DocumentListener updateOK_DL){
			super();
			this.setEditable(false);
			this.getDocument().addDocumentListener(updateOK_DL);
		}
	}

	/**
	 * OptionalSelectButton create a check box with an 
	 * attached listener and an optional
	 * text field with associated listener
	 * that is enabled/editable based on the selection
	 * of the check box.
	 * @author Kathe Todd-Brown
	 *
	 */
	@SuppressWarnings("serial")
	public static class OptionalSelectButton extends JCheckBox {
		
		public OptionalSelectButton(String s, ItemListener updateOK){
			super(s);
			this.addItemListener(updateOK);
		}
		
		public OptionalSelectButton(String s, JTextField target, 
				ItemListener updateOK){
			super(s);
			this.addItemListener(updateOK);
			this.addItemListener(new ActivateButton(this, target));
		}
		
		public OptionalSelectButton(String s, JTextField target, JButton browser, 
				ItemListener updateOK){
			super(s);
			this.addItemListener(updateOK);
			this.addItemListener(new ActivateButton(this, target, browser));
		}
		
		public OptionalSelectButton(String s, JTextField [] target, 
				ItemListener updateOK){
			super(s);
			this.addItemListener(updateOK);
			for(int i = 0; i < target.length ; i ++)
				this.addItemListener(new ActivateButton(this, target[i]));
		}
		public OptionalSelectButton(String s, JTextField [] target, JButton [] browser, 
				ItemListener updateOK){
			super(s);
			this.addItemListener(updateOK);
			for(int i = 0; i < target.length ; i ++)
				this.addItemListener(new ActivateButton(this, target[i], browser[i]));
		}
	}
	
	/**
	 * RequiredSelection creates a radio button that has an attached 
	 * action listener and
	 * option text field with associated listener
	 *  that is enabled based on the button state.
	 * @author Kathe Todd-Brown
	 *
	 */
	@SuppressWarnings("serial")
	public static class RequiredSelection extends JRadioButton {
		
		public RequiredSelection(String s, ItemListener updateOK){
			super(s);
			this.addItemListener(updateOK);
		}
		
		public RequiredSelection(String s, JTextField target, 
				ItemListener updateOK){
			super(s);
			this.addItemListener(updateOK);
			this.addItemListener(new ActivateButton(this, target));
		}
		
		public RequiredSelection(String s, JTextField target, JButton browser, 
				ItemListener updateOK){
			super(s);
			this.addItemListener(updateOK);
			this.addItemListener(new ActivateButton(this, target, browser));
		}
		
		public RequiredSelection(String s, JTextField [] target, 
				ItemListener updateOK){
			super(s);
			this.addItemListener(updateOK);
			for(int i = 0; i < target.length ; i ++)
				this.addItemListener(new ActivateButton(this, target[i]));
		}
		public RequiredSelection(String s, JTextField [] target, JButton [] browser, 
				ItemListener updateOK){
			super(s);
			this.addItemListener(updateOK);
			for(int i = 0; i < target.length ; i ++)
				this.addItemListener(new ActivateButton(this, target[i], browser[i]));
		}
	}

	/**
	 * ActivateButton wraps a toggle button with a textfield and
	 * button such that the enabled state of the text field and 
	 * button is dependent on the toggle button.
	 * @author Kathe Todd-Brown
	 *
	 */
	 static private class ActivateButton implements ItemListener{

		JButton targetButton;
		JTextField targetField;
		JToggleButton trigger;
		
		/**
		 * creates an actionListener that lets the user interact with 
		 * the given text field and button based on the state of the 
		 * given checkbox
		 * @param givenc JCheckBox that determins if the user has access
		 * @param givenf JTextField that user can/cannot interact with
		 * @param givenButton JButton that user can/cannot interact with
		 */
		public ActivateButton(JToggleButton givenc, JTextField givenf,
				JButton givenButton){
			targetButton = givenButton;
			targetField = givenf;
			trigger = givenc;
		}
		
		public ActivateButton(JToggleButton givenc,
				JButton givenButton){
			targetButton = givenButton;
			targetField = null;
			trigger = givenc;
		} 
		
		public ActivateButton(JToggleButton givenc, JTextField givenf){
			targetButton = null;
			targetField = givenf;
			trigger = givenc;
		}
		
		public void itemStateChanged(ItemEvent arg0) {
			if(targetButton != null)
				targetButton.setEnabled(trigger.isSelected());
			if(targetField != null)
				targetField.setEditable(trigger.isSelected());
			
		}
	}
	
	/**
	 * pickAFileButton creates a button that is tied to a 
	 * file browser.
	 * @author Kathe Todd-Brown
	 *
	 */
	@SuppressWarnings("serial")
	public static class pickAFileButton extends JButton{
		public pickAFileButton(String [] suffix, String discription, JTextField reciver, 
				MainFrame mf){
			super("Browse");
			this.addActionListener(new GetFile
					(new ModFilter(suffix, discription), reciver, null, mf));
		}
		
		public pickAFileButton(String [] suffix, String discription, JTextField reciver, 
				Forms mf){
			super("Browse");
			this.addActionListener(new GetFile
					(new ModFilter(suffix, discription), reciver, mf, mf.myFrame));
		}
		
		public pickAFileButton(String suffix, String discription, JTextField reciver, 
				Forms mf){
			super("Browse");
			this.addActionListener(new GetFile
					(new ModFilter(suffix, discription), reciver, mf, mf.myFrame));
		}
		public pickAFileButton(JTextField reciver, Forms mf){
			super("Browse");
			this.addActionListener(new GetFile
					(null, reciver, mf, mf.myFrame));
		}
		
		public pickAFileButton(JTextField reciver, MainFrame mf){
			super("Browse");
			this.addActionListener(new GetFile
					(null, reciver, null, mf));
		}
		
		public pickAFileButton(String suffix, String discription, JTextField reciver, 
				Forms mf, boolean enabled){
			super("Browse");
			setEnabled(enabled);
			this.addActionListener(new GetFile
					(new ModFilter(suffix, discription), reciver, mf, mf.myFrame));
		}
		public pickAFileButton(JTextField reciver, Forms mf, boolean enabled){
			super("Browse");
			setEnabled(enabled);
			this.addActionListener(new GetFile
					(null, reciver, mf, mf.myFrame));
		}
	}
	
	/**
	 * GetFile is an action that launches a file browser.
	 * @author Kathe Todd-Brown
	 *
	 */
	static class GetFile implements ActionListener{

		//filter that is used in the file chooser
		FileFilter filter;
		//text field that recieves the file name
		JTextField field;
		Forms currentForm;
		MainFrame myFrame;
		/**
		 * constructor that doesn't do much more then set the parameters
		 * @param givenfilter filter that is sued in the file chooser
		 * @param givenfield text field that recieves the file name
		 */
		GetFile(FileFilter givenfilter, JTextField givenfield, Forms form, MainFrame mf){
			filter = givenfilter;
			field = givenfield;
			currentForm = form;
			myFrame = mf;
		}
		
		
		public void actionPerformed(ActionEvent e) {
			if(myFrame.data.isRemote(true) ){
				if(currentForm == null){
					new RemoteFileChooser(myFrame, myFrame.data.getConn(), 
							field, myFrame.data.getRemoteFile(), 
							false);
				} else {
					new RemoteFileChooser(currentForm, myFrame.data.getConn(), 
						field, myFrame.data.getRemoteFile(), 
						false);
				}
			}else{
				//create a new dialog to pick the directory
				JFileChooser fc = new JFileChooser();
				//filters the files so that we only see .map files
				if(filter != null)
					fc.addChoosableFileFilter(filter);
				fc.setCurrentDirectory(myFrame.data.getRoot());
				//only look at directories and files
				fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
				//show the dialog and when it is OK-ed
				if(fc.showOpenDialog(currentForm) == JFileChooser.APPROVE_OPTION){
					//if a directory is selected then get out without setting
					//..anything
					if(fc.getSelectedFile().isDirectory())
						return;
					else{
						//otherwise we have a valid file and set it to the
						//...text box
						field.setText(fc.getSelectedFile().getAbsolutePath());
					}
				}
			}
		}
		
	}

	
	/**
	 * ModFilter filters for files of a given suffex. 
	 * Node that this filter only matches one file
	 * extention at a time.
	 *
	 */
	static class ModFilter extends FileFilter{
		String [] suffex;
		String disc;
		
		
		
		/**
		 * constructor that doesn't do much more then set the parameters
		 * @param s string used to mach the suffex of the files
		 * @param d string used as a discription for the filter
		 */
		public ModFilter(String s, String d){
			suffex = new String [] {s};
			disc = d;
		}
		
		public ModFilter(String [] s, String d){
			suffex = s;
			disc = d;
		}
		
		public boolean accept(File f) {
			boolean ans = f.isDirectory();
			
			for(int i = 0; i < suffex.length; i++){
				ans = ans || f.getName().endsWith(suffex[i]);
			}
			
			return (ans);
		}
		public String getDescription() {
			return disc;
		}
	}
	
	
	
	/**
	 * quoteFilename adds " to the beiging and end of a 
	 * string if it is not already there
	 * @param fileName a string
	 * @return the input string with \" added to it if it
	 * is not already there
	 */
	public static String quoteFilename(String fileName){
		
		if(fileName.startsWith("\"") && fileName.endsWith("\"")){
			return fileName;
		}
			
		if(fileName.contains(" "))
			fileName = "\"" + fileName + "\"";
		return fileName;
	}
	
	/**
	 * makeSpacesHard replaces all spaces with backslash-space or " " -> "\ ".
	 * @param file some string that may or may not have spaces
	 * @return the same string but with the spaces now "hard"
	 */
	public static String makeSpacesHard(String file){
		String given = file;
		if(given.startsWith("\"") && given.endsWith("\"")){
			System.err.println("Trying to force to a filename that have \" !");
			return given;
		}
		
		if(given.contains(" ")){
			String [] temp = given.split("\\s");
			given = "";
			for(int i = 0; i < temp.length - 1 ; i ++){
				given += temp[i] + "\\ ";
			}
			given += temp[temp.length -1];
		}
			
		return given;
		
	}
	
	/**
	 * makeFileSpacesHard takes a command with files 
	 * in "" and splits them and makes the spaces in the files
	 * hard ([ ] -> [\ ]).
	 * @param command a string with "" around the segments you want
	 * to make hard.
	 * @return a string where the desired segments now have hard 
	 * spaces.
	 */
	public static String makeQuotedSpacesHard(String command){
		String [] temp  = stripAndSplit(command);
		String ans = "";
		for(int i = 0; i < temp.length; i++){
			temp[i] = makeSpacesHard(temp[i]);
			ans += temp[i] + " ";
		}
		return ans;
	}
	
	/**
	 * fileName takes a full file name seperated by either \ or / and return
	 * the name and extention of that full name
	 * @param file a full file name
	 * @return the file name striped of the directory info
	 */
	public static String fileName(String file){
		if(file == null)
			return null;
		
		String [] temp = null;
		String tfile = file;
		
		if(tfile.endsWith("\"")){
			tfile = tfile.substring(0, tfile.length()-1);
		}
		
		if(tfile.contains("/")){
			temp = tfile.split("/");
		}
		else if(tfile.contains("\\")){
			temp = tfile.split("\\\\");
			
		} else {
			return tfile;
		}
		return temp[temp.length -1];
	}
	
	/**
	 * stripAndSplit splits string based on spacing outside of 
	 * quotes then strip the quotes off.
	 * @param given a string
	 * @return an array is populated by the split given string
	 */
	public static String [] stripAndSplit(String given){
		Vector <String> temp = new Vector<String>();
		temp.add("");
		given = given.trim();
		//given = given.replace("\\s", " ");
		int currentTemp = 0;
		boolean insideQuotes = false;
		//go through the entier string
		for(int i = 0; i < given.length(); i++){
			//flag when we pass "
			if(given.charAt(i) == ('\"'))
				insideQuotes = ! insideQuotes;
			else{
				//if we aren't inside quotes
				if(insideQuotes == false 
						&& given.charAt(i) == ' '){
						//check to make sure it's not empty before
						//...we move on
						if(!temp.get(currentTemp).equals("")){
							currentTemp ++;
							temp.add(currentTemp, "");
						}
					}
				else
					//add the characture to the current
					temp.set(currentTemp, 
						temp.get(currentTemp) + given.charAt(i));
			}
		}
		//create an array of the correct size
		String [] ans = new String [temp.size()];
		//populate it with the processed given string
		for(int i = 0; i < temp.size(); i++)
			ans[i] = temp.get(i);
		//return that array
		return ans;
	}
}
