package edu.harvard.mgh.purcell.gplink.forms;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.ButtonGroup;
import javax.swing.border.TitledBorder;

import edu.harvard.mgh.purcell.gplink.base_form.*;
import edu.harvard.mgh.purcell.gplink.mainGUI.*;

/**
 * A form that allows the user to create a Haplotype-based C/C test plink
 * command line
 * @author Kathe Todd-Brown
 *
 */
@SuppressWarnings("serial")
public class HapCCAssocForm extends Forms {

	Forms.myRadioButton hapListButton, hapWindowButton;
	Forms.myJTextField hapListText, hapWindowText;
	Forms.pickAFileButton hapListBrowse;
	
	/**
	 * Create a Haplotype-based C/C test
	 * @param mf main frame the form is attached to
	 */
	public HapCCAssocForm(MainFrame mf) {
		super(mf, "Haplotype-based C/C tests");
	}

	@Override
	protected void createBody() {
		
		body.setLayout(new GridBagLayout());
		body.setBorder(new TitledBorder("Haplotype specification"));
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		body.add(hapListButton, c);
		c.gridx = 1;
		c.weightx = 1;
		body.add(hapListText,c);
		c.weightx = 0;
		c.gridx = 2;
		body.add(hapListBrowse, c);
		c.gridy = 1;
		c.gridx = 0;
		body.add(hapWindowButton, c);
		c.gridwidth = 2;
		c.gridx = 1;
		c.weightx = 1;
		body.add(hapWindowText, c);
		
	}

	@Override
	protected void initalize() {
		
		hapWindowText = new myJTextField(10);
		hapListText = new myJTextField(20);
		
		hapListBrowse = new pickAFileButton(".hlist", "Haplotype list (*.hlist)", 
				hapListText);
		
		hapWindowButton = new myRadioButton("Sliding window (--hap-assoc --hap-window)", 
				hapWindowText);
		hapListButton = new myRadioButton("Haplotype list (--hap-assoc --hap)", 
				hapListText, hapListBrowse);
		hapListButton.setSelected(true);
		
		ButtonGroup options = new ButtonGroup();
		options.add(hapWindowButton);
		options.add(hapListButton);
	}

	@Override
	protected boolean isBodyValid() {
		if( hapListButton.isSelected() 
				&& hapListText.getText().length() == 0){
//			information.setText("Must select a file associated with the --hap flag.");
			return false;
			
		}
		if( hapWindowButton.isSelected() 
				 && !hapWindowText.getText().matches("\\d+")){
//			information.setText("Must enter a numeric value associated with " +
//					"the --hap-window flag.");
			return false;
		}
		return true;
	}

	@Override
	protected String processBody() {
		String ans = " --hap-assoc";
		if(hapListButton.isSelected())
			ans += " --hap " + GeneralUtil.quoteFilename(hapListText.getText());
		if(hapWindowButton.isSelected())
			ans += " --hap-window " + hapWindowText.getText();
		return ans;
	}

}
