package edu.harvard.mgh.purcell.gplink.forms;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JSeparator;
import javax.swing.SwingConstants;

import edu.harvard.mgh.purcell.gplink.base_form.Forms;
import edu.harvard.mgh.purcell.gplink.mainGUI.*;

@SuppressWarnings("serial")
public class IBSForm extends Forms {
	
	Forms.myRadioButton homSNPButton, homKBButton;
	Forms.myCheckBox		
		homGroupButton, homMatchButton, homHetButton, homVerboseButton;
	
	Forms.myJTextField homSNPText, homKBText, homMatchText, homHetText;

	public IBSForm(MainFrame mf) {
		super(mf, "Runs of homozygosity");
	}

	@Override
	protected void createBody() {

		body.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		body.add(homKBButton, c);
		c.gridx = 1;
		c.weightx = 1;
		body.add(homKBText, c);
		
		c.weightx = 0;
		c.gridy = 1;
		c.gridx = 0;
		body.add(homSNPButton, c);
		
		c.gridx = 1;
		c.weightx = 1;
		body.add(homSNPText,c);
		
		c.gridx = 0;
		c.gridy = 2;
		c.gridwidth = 2;
		body.add(new JSeparator(SwingConstants.HORIZONTAL), c);
		
		c.gridwidth = 1;
		c.weightx = 0;
		c.gridy = 3;
		c.gridx = 0;
		body.add(homHetButton, c);
		c.gridx = 1;
		body.add(homHetText, c);
		
		c.gridy = 4;
		c.gridx = 0;
		body.add(homGroupButton, c);
		
		c.gridy = 5;
		body.add(homMatchButton, c);
		
		c.gridx = 1;
		body.add(homMatchText, c);
		
		c.gridy = 6;
		c.gridx = 0;
		body.add(homVerboseButton, c);
	

	}

	@Override
	protected String processBody() {
		String ans = "";

		if(homKBButton.isSelected())
			ans += " --homozyg-kb " + homKBText.getText();
		if(homSNPButton.isSelected())
			ans += " --homozyg-snp " + homSNPText.getText();
		if(homHetButton.isSelected())
			ans += " --homozyg-het " + homHetText.getText();
		if(homGroupButton.isSelected())
			ans += " --homozyg-group";
		if(homMatchButton.isSelected())
			ans += " --homozyg-match " + homMatchText.getText();
		if(homVerboseButton.isSelected())
			ans += " --homozyg-verbose";
		ans = ans.replaceFirst(" ", "");
		
		return ans;
	}

	@Override
	protected boolean isBodyValid() {
		if(!(homSNPButton.isSelected() 
			|| homKBButton.isSelected())){
//			information.setText("Either the --homozyg-kb and/or --homozyg-snp must" +
//					"be selected.");
			return false;
		}
		if(homKBButton.isSelected() && !homKBText.getText().matches("\\d+")){
//			information.setText("Enter the associated value for --homozyg-kb.");
			return false;
		}
			
		if(homSNPButton.isSelected() && !homSNPText.getText().matches("\\d+")){
//			information.setText("Enter the associated value for --homozyg-snp.");
			return false;
		}
		if(homHetButton.isSelected() && !homHetText.getText().matches("\\d+")){
//			information.setText("Enter the associated value for --homozyg-het.");
			return false;
		}
		if(homMatchButton.isSelected() && 
				!homMatchText.getText().matches("\\d*\\.?\\d+")){
//			information.setText("Enter the associated value for --homozyg-match.");
			return false;
		}
		
		return true;
	}

	@Override
	protected void initalize() {

		homSNPText = new myJTextField(4);
		homKBText = new myJTextField(4);
		homHetText = new myJTextField(4);
		homMatchText = new myJTextField(4);
		
		homSNPButton = new myRadioButton("Homozygous run length, SNPs" +
				" (--homozyg-snp)", homSNPText);
		homKBButton = new myRadioButton("Homozygous run length, kb " +
				"(--homozyg-kb)", homKBText); 
		
		homKBButton.setSelected(true);
		
		homHetButton = new myCheckBox("Heterozygotes allowed in " +
				"homozygous run (--homozyg-het)", homHetText);
		homGroupButton = new myCheckBox("Group overlapping segments " +
				"(--homozyg-group)");
		homMatchButton = new myCheckBox("Threshold for allelic match " +
				"(--homozyg-match)", homMatchText);
		homVerboseButton = new myCheckBox("Verbose segment listing " +
				"(--homozyg-verbose)");

	}

}
