package edu.harvard.mgh.purcell.gplink.forms;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JLabel;
import javax.swing.JPanel;

import edu.harvard.mgh.purcell.gplink.base_form.Forms;
import edu.harvard.mgh.purcell.gplink.base_form.GeneralUtil;
import edu.harvard.mgh.purcell.gplink.mainGUI.*;

@SuppressWarnings("serial")
public class NeighForm extends Forms {

	Forms.myJTextField neigh1Text, neigh2Text, genomicText;
	Forms.myCheckBox neighButton;
	
	public NeighForm(MainFrame mf) {
		super(mf, "Nearest Neighbour");
		
	}

	@Override
	protected void createBody() {
		body.setLayout(new GridBagLayout());
		
		JPanel genoPanel = new JPanel();
		genoPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.ipadx = 5;
		c.weightx = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		genoPanel.add(new JLabel("IBS distance file (--read-genome)"), c);
		c.weightx = 1;
		c.gridx = 1;
		genoPanel.add(genomicText, c);
		c.gridx = 2;
		c.weightx = 0;
		genoPanel.add(new Forms.pickAFileButton(".genome", 
				"GENOME", genomicText), c);
		
		JPanel neighPanel = new JPanel();
		neighPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		c = new GridBagConstraints();
		neighPanel.add(neighButton, c);
		c.gridx = 1;
		neighPanel.add(neigh1Text, c);
		c.gridx = 2;
		neighPanel.add(neigh2Text, c);
		
		c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weighty = 1;
		c.weightx = 1;
		body.add(genoPanel, c);
		c.gridy = 1;
		c.fill = GridBagConstraints.VERTICAL;
		c.anchor = GridBagConstraints.WEST;
		body.add(neighPanel, c);
		
	}

	@Override
	protected String processBody() {
		String ans = "";
		ans += "--cluster ";
		ans += "--read-genome " + GeneralUtil.quoteFilename(genomicText.getText());
		if(neighButton.isSelected())
			ans += " --neighbour " 
				+ neigh1Text.getText() 
				+ " " + neigh2Text.getText();
		
		return ans;
	}

	@Override
	protected boolean isBodyValid() {
		if(genomicText.getText().length() < 0){
//			information.setText("Must select a genome file to read from.");
			return false;
		}
		if (neighButton.isSelected() 
				&& ! ( neigh1Text.getText().matches("\\d+")
					  && neigh2Text.getText().matches("\\d+") )){
//			information.setText("Must set valid --neighbour parameters.");
			return false;
		}
		
		return true;
		/*
		if(new File(genomicText.getText()).exists())
			if(neighButton.isSelected())
				if( ! ( neigh1Text.getText().equals("")
					|| neigh2Text.getText().equals("") ))
					return true;

		return false;
		*/
	}

	@Override
	protected void initalize() {
		genomicText = new myJTextField(10);
		neigh1Text = new myJTextField(4);
		neigh2Text = new myJTextField(4);
		
		myJTextField [] temp = {neigh1Text, neigh2Text};
		
		neighButton = new Forms.myCheckBox("Outlier statistics", temp);
	}

}
